@@ -22,33 +22,55 @@
 #endif
 
 #include "appdata.h"
+#include "stats/node.h"
 
 #define ETHERAPE_GLADE_FILE "etherape.glade"	/* glade 3 file */
 
+struct appdata_struct appdata;
+
 void appdata_init(struct appdata_struct *p)
 {
-  gettimeofday (&p->now, NULL);
+  gettimeofday(&p->gui_now, NULL);
+  memset(&p->now, 0, sizeof(p->now));
 
   p->xml = NULL;
   p->app1 = NULL;
   p->statusbar = NULL;
 
   p->glade_file = NULL;
-  p->input_file = NULL;
   p->export_file = NULL;
   p->export_file_final = NULL;
   p->export_file_signal = NULL;
-  p->interface = NULL;
+
+  p->source.type = ST_LIVE;
+  p->source.interface = NULL;
 
   p->mode = IP;
   p->node_limit = -1;
   p->debug_mask = (G_LOG_LEVEL_MASK & ~(G_LOG_LEVEL_DEBUG | G_LOG_LEVEL_INFO));
   p->min_delay = 0;
   p->max_delay = G_MAXULONG;
+  p->stationary_layout = FALSE;
 
   p->n_packets = 0;
   p->total_mem_packets = 0;
   p->request_dump = FALSE;
+
+  p->column_patterns = NULL;
+}
+
+void appdata_clear_source(struct appdata_struct *p)
+{
+  if (p->source.type == ST_LIVE)
+    {
+      g_free(p->source.interface);
+      p->source.interface = NULL;
+    }
+  else
+    {
+      g_free(p->source.file);
+      p->source.file = NULL;
+    }
 }
 
 gboolean appdata_init_glade(gchar *new_glade_file)
@@ -75,12 +97,11 @@ gboolean appdata_init_glade(gchar *new_glade_file)
 /* releases all memory allocated for internal fields */
 void appdata_free(struct appdata_struct *p)
 {
+  appdata_clear_source(p);
+
   g_free(p->glade_file);
   p->glade_file = NULL;
 
-  g_free(p->input_file);
-  p->input_file = NULL;
-
   g_free(p->export_file);
   p->export_file = NULL;
 
@@ -90,8 +111,14 @@ void appdata_free(struct appdata_struct *p)
   g_free(p->export_file_signal);
   p->export_file_signal = NULL;
 
-  g_free(p->interface);
-  p->interface=NULL;
+  if (p->column_patterns) {
+    int pos;
+    for (pos = 0; pos < p->column_patterns->len; ++pos) {
+       free_nodeset_spec_list(g_ptr_array_index(p->column_patterns, pos));
+    }
+    g_ptr_array_free(p->column_patterns, TRUE);
+    p->column_patterns = NULL;
+  }
 
   /* no need to free glade widgets ... */
 }
