@@ -26,21 +26,27 @@
 #include "appdata.h"
 #include "diagram.h"
 #include "pref_dialog.h"
-#include "node.h"
+#include "stats/node.h"
 #include "info_windows.h"
-#include "protocols.h"
+#include "stats/protocols.h"
 #include "datastructs.h"
-#include "ip-cache.h"
+#include "names/ip-cache.h"
 #include "menus.h"
-#include "capture.h"
-#include "conversations.h"
+#include "capture/capctl.h"
+#include "stats/conversations.h"
 #include "preferences.h"
 #include "export.h"
+#include "stats/util.h"
 
 /* maximum node and link size */
 #define MAX_NODE_SIZE 5000
 #define MAX_LINK_SIZE (MAX_NODE_SIZE/4)
 
+gboolean already_updating;
+gboolean stop_requested;
+
+static GnomeCanvasItem *pcap_stats_text_item = NULL;
+static GnomeCanvasGroup *pcap_stats_text_group = NULL;
 
 /***************************************************************************
  *
@@ -57,6 +63,10 @@ typedef struct
   gboolean is_new;
   gboolean shown;		/* True if it is to be displayed. */
   gboolean centered;            /* true if is a center node */
+
+  /* For '-P' mode (columnar layout) */
+  guint column; /* Which column this goes in */
+  guint column_idx; /* Which position within its column this node is */
 }
 canvas_node_t;
 static gint canvas_node_compare(const node_id_t *a, const node_id_t *b, 
@@ -87,13 +97,20 @@ static gint canvas_link_update(link_id_t * link_id,
 				 canvas_link_t * canvas_link,
 				 GList ** delete_list);
 
-
-typedef struct 
+struct node_ring
 {
-  GtkWidget * canvas;
   gfloat angle;
   guint node_i;
   guint n_nodes;
+};
+
+typedef struct
+{
+  GtkWidget * canvas;
+
+  struct node_ring outer;
+  struct node_ring center;
+
   gdouble xmin;
   gdouble ymin;
   gdouble xmax;
@@ -102,11 +119,31 @@ typedef struct
   gdouble y_rad_max;
   gdouble x_inner_rad_max;
   gdouble y_inner_rad_max;
-  
+
+  guint *column_populations;
 } reposition_node_t;
 
+/***************************************************************************
+ *
+ * canvas_background_t definition and methods
+ *
+ **************************************************************************/
 
+typedef struct
+{
+  GnomeCanvasGroup *group;
+  gboolean use_image;
 
+  GdkColor color;
+  struct
+  {
+    GdkPixbuf *image;
+    gchar *path;
+    GnomeCanvasItem *item;
+  } image;
+} canvas_background_t;
+
+static void init_canvas_background(GtkWidget *canvas);
 
 /***************************************************************************
  *
@@ -114,19 +151,15 @@ typedef struct
  *
  **************************************************************************/
 
-
 static GTree *canvas_nodes;	/* We don't use the nodes tree directly in order to 
 				 * separate data from presentation: that is, we need to
 				 * keep a list of CanvasItems, but we do not want to keep
 				 * that info on the nodes tree itself */
 static GTree *canvas_links;	/* See canvas_nodes */
-
 static guint known_protocols = 0;
-
-
+static canvas_background_t canvas_background;
 static gboolean is_idle = FALSE;
 static guint displayed_nodes;
-static GdkColor black_color;
 static gboolean need_reposition = TRUE;	/* Force a diagram relayout */
 static gboolean need_font_refresh = TRUE;/* Force font refresh during layout */
 static gint diagram_timeout;	/* Descriptor of the diagram timeout function
@@ -141,6 +174,7 @@ static long canvas_obj_count = 0; /* counter of canvas objects */
  **************************************************************************/
 static void diagram_update_nodes(GtkWidget * canvas); /* updates ALL nodes */
 static void diagram_update_links(GtkWidget * canvas); /* updates ALL links */
+static void diagram_update_background_image(GtkWidget *canvas); /* updates background image */
 
 static void check_new_protocol (GtkWidget *prot_table, const protostack_t *pstk);
 static gint check_new_node (node_t * node, GtkWidget * canvas);
@@ -155,6 +189,9 @@ static gint traffic_compare (gconstpointer a, gconstpointer b);
 static gint reposition_canvas_nodes (node_id_t * node_id,
 				     canvas_node_t * canvas_node,
 				     reposition_node_t *data);
+static gint reposition_canvas_nodes_prep(const node_id_t * node_id,
+                                         canvas_node_t * canvas_node,
+                                         reposition_node_t *data);
 static gint check_new_link (link_id_t * link_id,
 			    link_t * link, GtkWidget * canvas);
 static gdouble get_node_size (gdouble average);
@@ -163,6 +200,8 @@ static gint link_item_event (GnomeCanvasItem * item,
 			     GdkEvent * event, canvas_link_t * canvas_link);
 static gint node_item_event (GnomeCanvasItem * item,
 			     GdkEvent * event, canvas_node_t * canvas_node);
+static gint pcap_stats_text_item_event(GnomeCanvasItem *item, GdkEvent *event,
+                                       void *unused);
 static void update_legend(void);
 static void draw_oneside_link(double xs, double ys, double xd, double yd,
                               const basic_stats_t *link_data, 
@@ -170,6 +209,7 @@ static void draw_oneside_link(double xs, double ys, double xd, double yd,
 static void init_reposition(reposition_node_t *data,
                             GtkWidget * canvas, 
                             guint total_nodes);
+static void clear_reposition(reposition_node_t *rdata);
 
 
 void ask_reposition(gboolean r_font)
@@ -224,8 +264,9 @@ static void addref_canvas_obj(GObject *obj)
 void
 init_diagram (GladeXML *xml)
 {
-  GtkStyle *style;
   GtkWidget *canvas;
+  GtkWidget *viewport;
+  GtkStyle *style;
 
   /* Creates trees */
   canvas_nodes = g_tree_new_full ( (GCompareDataFunc)canvas_node_compare, 
@@ -234,19 +275,36 @@ init_diagram (GladeXML *xml)
                             NULL, NULL, (GDestroyNotify)canvas_link_delete);
 
   initialize_pref_controls();
-  
-  /* Sets canvas background to black */
+
+  /* Initialize background image */
   canvas = glade_xml_get_widget (appdata.xml, "canvas1");
+  init_canvas_background(canvas);
 
-  gdk_color_parse ("black", &black_color);
-  gdk_colormap_alloc_color (gdk_colormap_get_system (), &black_color,
-			    TRUE, TRUE);
+  /* Make legend background color match main display background color */
   style = gtk_style_new ();
-  style->bg[GTK_STATE_NORMAL] = black_color;
-  style->base[GTK_STATE_NORMAL] = black_color;
+  style->bg[GTK_STATE_NORMAL] = canvas_background.color;
+  style->base[GTK_STATE_NORMAL] = canvas_background.color;
 
-  gtk_widget_set_style (canvas, style);
-  gtk_style_set_background (canvas->style, canvas->window, GTK_STATE_NORMAL);
+  /* Set protocol legend background to black */
+  viewport = glade_xml_get_widget (appdata.xml, "legend_viewport");
+  gtk_widget_set_style (viewport, style);
+  gtk_style_set_background (viewport->style, viewport->window, GTK_STATE_NORMAL);
+
+  /* Create pcap stats text */
+  pcap_stats_text_group = gnome_canvas_root(GNOME_CANVAS(canvas));
+  pcap_stats_text_group = GNOME_CANVAS_GROUP(gnome_canvas_item_new(pcap_stats_text_group,
+                                                                   GNOME_TYPE_CANVAS_GROUP,
+                                                                   "x", 0.0,
+                                                                   "y", 0.0,
+                                                                   NULL));
+  addref_canvas_obj(G_OBJECT(pcap_stats_text_group));
+
+  pcap_stats_text_item = gnome_canvas_item_new(pcap_stats_text_group,
+                                               GNOME_TYPE_CANVAS_TEXT, NULL);
+  addref_canvas_obj(G_OBJECT(pcap_stats_text_item));
+
+  g_signal_connect(G_OBJECT(pcap_stats_text_item), "event",
+                   (GtkSignalFunc)pcap_stats_text_item_event, NULL);
 
   /* Initialize the known_protocols table */
   delete_gui_protocols ();
@@ -256,6 +314,112 @@ init_diagram (GladeXML *xml)
   stop_requested = FALSE;
 }				/* init_diagram */
 
+/*
+ * Initialize the canvas_background structure.
+ * Create a new group and add it to the root group.
+ * Create a new GnomeCanvasPixbufItem and add it to the new group.
+ */
+static void init_canvas_background(GtkWidget *canvas)
+{
+  GnomeCanvasGroup *group;
+  GtkStyle *style;
+
+  group = gnome_canvas_root(GNOME_CANVAS(canvas));
+  canvas_background.group = GNOME_CANVAS_GROUP(gnome_canvas_item_new(group,
+                                                                     GNOME_TYPE_CANVAS_GROUP,
+                                                                     "x", 0.0, "y", 0.0,
+                                                                     NULL));
+  addref_canvas_obj(G_OBJECT(group));
+
+  canvas_background.image.item = gnome_canvas_item_new(canvas_background.group,
+                                                       gnome_canvas_pixbuf_get_type(),
+                                                       "anchor", GTK_ANCHOR_NW,
+                                                       NULL);
+  addref_canvas_obj(G_OBJECT(canvas_background.image.item));
+
+  canvas_background.use_image = pref.bck_image_enabled;
+  canvas_background.image.path = g_strdup(pref.bck_image_path);
+
+  gdk_color_parse("black", &canvas_background.color);
+  gdk_colormap_alloc_color(gdk_colormap_get_system(), &canvas_background.color, TRUE, TRUE);
+
+  style = gtk_style_new();
+  style->bg[GTK_STATE_NORMAL] = canvas_background.color;
+  style->base[GTK_STATE_NORMAL] = canvas_background.color;
+
+  gtk_widget_set_style(canvas, style);
+  gtk_style_set_background(canvas->style, canvas->window, GTK_STATE_NORMAL);
+
+  /**
+   * Have to set the scrolling region to the real allocated size of the canvas
+   * If not the image won't scale well in the redraw_canvas_background
+   * function until the on_canvas1_size_allocate callback is called
+   */
+  gnome_canvas_set_scroll_region(GNOME_CANVAS(canvas),
+                                 -(canvas->allocation.width)/2, -(canvas->allocation.height)/2,
+                                 (canvas->allocation.width)/2, (canvas->allocation.height)/2);
+}
+
+/*
+ * Called by on_canvas1_size_allocate.
+ * Try to load image from path.
+ * On success image is scaled to the canvas size and printed.
+ * On error a blank black image is made and printed.
+ */
+void redraw_canvas_background(GtkWidget *canvas)
+{
+  GError* error = NULL;
+  GtkAllocation canvas_size;
+
+  if (canvas_background.use_image) {
+    /* Get canvas dimensions */
+    canvas_size.width = canvas->allocation.width;
+    canvas_size.height = canvas->allocation.height;
+
+    if (canvas_background.image.image) {
+      g_object_unref(G_OBJECT(canvas_background.image.image));
+      canvas_background.image.image = NULL;
+    }
+
+    if (canvas_background.image.path && strlen(canvas_background.image.path))
+      canvas_background.image.image = gdk_pixbuf_new_from_file_at_scale(canvas_background.image.path,
+                                                                        canvas_size.width, canvas_size.height,
+                                                                        FALSE, &error);
+
+    gnome_canvas_item_set(canvas_background.image.item,
+                          "pixbuf", canvas_background.image.image,
+                          "x", (double)-canvas_size.width/2,
+                          "y", (double)-canvas_size.height/2,
+                          NULL);
+
+    gnome_canvas_item_show(canvas_background.image.item);
+  } else {
+    gnome_canvas_item_hide(canvas_background.image.item);
+  }
+}
+
+/*
+ * Update the background image.  Load new image if user selected another path
+ * in preferences.  Place the background image behind the nodes and links.
+ */
+static void diagram_update_background_image(GtkWidget *canvas)
+{
+  /*
+   * If the background image enable toggle or the image path has changed, we
+   * need to update the background.
+   */
+  if (pref.bck_image_enabled != canvas_background.use_image
+      || g_strcmp0(canvas_background.image.path, pref.bck_image_path)) {
+    canvas_background.use_image = pref.bck_image_enabled;
+    g_free(canvas_background.image.path);
+    canvas_background.image.path = g_strdup(pref.bck_image_path);
+    redraw_canvas_background(canvas);
+  }
+
+  /* Draw the background first */
+  gnome_canvas_item_lower_to_bottom(GNOME_CANVAS_ITEM(canvas_background.group));
+  gnome_canvas_item_request_update(canvas_background.image.item);
+}
 
 void
 destroying_timeout (gpointer data)
@@ -355,9 +519,20 @@ diagram_update_nodes(GtkWidget * canvas)
     {
       reposition_node_t rdata;
       init_reposition(&rdata, canvas, displayed_nodes);
+
+      g_tree_foreach(canvas_nodes,
+                    (GTraverseFunc) reposition_canvas_nodes_prep,
+                    &rdata);
+
+      rdata.center.node_i = rdata.center.n_nodes;
+      rdata.outer.node_i = rdata.outer.n_nodes;
+
       g_tree_foreach(canvas_nodes,
                     (GTraverseFunc) reposition_canvas_nodes,
                     &rdata);
+
+      clear_reposition(&rdata);
+
       need_reposition = FALSE;
       need_font_refresh = FALSE;
     }
@@ -388,6 +563,83 @@ diagram_update_links(GtkWidget * canvas)
   g_list_free(delete_list);
 }
 
+/* Return a g_malloc()ed string of libpcap stats counters */
+static gchar *get_pcap_stats_string(void)
+{
+  struct pcap_stat stats;
+
+  if (appdata.source.type == ST_FILE)
+    return g_strdup(_("(Capture statistics unavailable in offline mode.)"));
+
+  get_capture_stats(&stats);
+
+  return g_strdup_printf("%-12s %12u\n%-12s %12u\n%-12s %12u", "recv:",
+                         stats.ps_recv, "drop:", stats.ps_drop,
+                         "ifdrop:", stats.ps_ifdrop);
+}
+
+/* Update libpcap stats counters display */
+static void update_pcap_stats_text(GtkWidget *canvas)
+{
+  gdouble xmin, xmax, ymin, ymax, xpos, ypos;
+  GtkAnchorType anchor;
+  gchar *tmpstr;
+
+  if (pref.pcap_stats_pos == STATSPOS_NONE)
+    {
+      gnome_canvas_item_hide(pcap_stats_text_item);
+      return;
+    }
+  else
+    gnome_canvas_item_show(pcap_stats_text_item);
+
+  if (get_capture_status() != PLAY)
+    return;
+
+  gnome_canvas_get_scroll_region(GNOME_CANVAS(canvas), &xmin, &ymin, &xmax, &ymax);
+
+  switch (pref.pcap_stats_pos)
+    {
+    case STATSPOS_UPPER_LEFT:
+      xpos = xmin;
+      ypos = ymin;
+      anchor = GTK_ANCHOR_NW;
+      break;
+
+    case STATSPOS_UPPER_RIGHT:
+      xpos = xmax;
+      ypos = ymin;
+      anchor = GTK_ANCHOR_NE;
+      break;
+
+    case STATSPOS_LOWER_LEFT:
+      xpos = xmin;
+      ypos = ymax;
+      anchor = GTK_ANCHOR_SW;
+      break;
+
+    case STATSPOS_LOWER_RIGHT:
+      xpos = xmax;
+      ypos = ymax;
+      anchor = GTK_ANCHOR_SE;
+      break;
+
+    default:
+      g_error(_("Bogus statspos_t (%d) pref.pcap_stats_pos"), pref.pcap_stats_pos);
+      return;
+    }
+
+  tmpstr = get_pcap_stats_string();
+  gnome_canvas_item_set(pcap_stats_text_item,
+                        "text", tmpstr,
+                        "x", xpos, "y", ypos,
+                        "font", pref.fontname,
+                        "fill_color", pref.text_color,
+                        "anchor", anchor,
+                        NULL);
+  g_free(tmpstr);
+}
+
 /* Refreshes the diagram. Called each refresh_period ms
  * 1. Checks for new protocols and displays them
  * 2. Updates nodes looks
@@ -397,7 +649,7 @@ guint update_diagram(GtkWidget * canvas)
 {
   static struct timeval last_refresh_time = { 0, 0 };
   double diffms;
-  enum status_t status;
+  capstatus_t status;
 
   /* if requested and enabled, dump to xml */
   if (appdata.request_dump && appdata.export_file_signal)
@@ -432,7 +684,7 @@ guint update_diagram(GtkWidget * canvas)
     }
 
   already_updating = TRUE;
-  gettimeofday (&appdata.now, NULL);
+  gettimeofday(&appdata.gui_now, NULL);
 
   /* update nodes */
   diagram_update_nodes(canvas);
@@ -440,6 +692,9 @@ guint update_diagram(GtkWidget * canvas)
   /* update links */
   diagram_update_links(canvas);
 
+  /* update background image */
+  diagram_update_background_image(canvas);
+  
   /* Update protocol information */
   protocol_summary_update_all();
 
@@ -449,19 +704,21 @@ guint update_diagram(GtkWidget * canvas)
   /* Now update info windows */
   update_info_windows ();
 
+  update_pcap_stats_text(canvas);
+
   /* With this we make sure that we don't overload the
    * CPU with redraws */
 
   if ((last_refresh_time.tv_sec == 0) && (last_refresh_time.tv_usec == 0))
-    last_refresh_time = appdata.now;
+    last_refresh_time = appdata.gui_now;
 
   /* Force redraw */
   while (gtk_events_pending ())
     gtk_main_iteration ();
 
-  gettimeofday (&appdata.now, NULL);
-  diffms = substract_times_ms(&appdata.now, &last_refresh_time);
-  last_refresh_time = appdata.now;
+  gettimeofday(&appdata.gui_now, NULL);
+  diffms = subtract_times_ms(&appdata.gui_now, &last_refresh_time);
+  last_refresh_time = appdata.gui_now;
 
   already_updating = FALSE;
 
@@ -815,7 +1072,7 @@ display_node (node_t * node)
   if (!node)
     return FALSE;
 
-  diffms = substract_times_ms(&appdata.now, &node->node_stats.stats.last_time);
+  diffms = subtract_times_ms(&appdata.now, &node->node_stats.stats.last_time);
 
   /* There are problems if a canvas_node is deleted if it still
    * has packets, so we have to check that as well */
@@ -918,24 +1175,31 @@ traffic_compare (gconstpointer a, gconstpointer b)
 
 }				/* traffic_compare */
 
-
 /* initialize reposition struct */
 static void init_reposition(reposition_node_t *data,
-                            GtkWidget * canvas, 
+                            GtkWidget * canvas,
                             guint total_nodes)
 {
   gdouble text_compensation = 50;
 
   data->canvas = canvas;
-  data->angle = 0.0;
-  data->node_i = 0;
-  data->n_nodes = 0;
-  data->n_nodes = total_nodes;
-  data->node_i = total_nodes;
+  memset(&data->center, 0, sizeof(data->center));
+  memset(&data->outer, 0, sizeof(data->outer));
 
-  gnome_canvas_get_scroll_region (GNOME_CANVAS (canvas),
-				  &data->xmin, &data->ymin, 
-                                  &data->xmax, &data->ymax);
+  /*
+   * Offset the starting angle on the center ring so that when there are
+   * relatively few nodes displayed (e.g. 4 central and 4 outer) the links
+   * obscure things less (by not overlapping node labels and other links).
+   */
+  data->center.angle += M_PI / 4.0;
+
+  if (appdata.column_patterns)
+    data->column_populations = g_malloc0(appdata.column_patterns->len + 1 *
+                                           sizeof(*data->column_populations));
+
+  gnome_canvas_get_scroll_region(GNOME_CANVAS (canvas),
+                                 &data->xmin, &data->ymin,
+                                 &data->xmax, &data->ymax);
 
 
   data->xmin += text_compensation;
@@ -949,6 +1213,78 @@ static void init_reposition(reposition_node_t *data,
   data->y_inner_rad_max = data->y_rad_max / 2;
 }
 
+static void clear_reposition(reposition_node_t *rdata)
+{
+  if (appdata.column_patterns)
+    g_free(rdata->column_populations);
+}
+
+static guint find_node_column(node_t *node)
+{
+  guint i;
+
+  /* This should only be called if we're in columnar-positioning mode */
+  g_assert(appdata.column_patterns);
+
+  for (i = 0; i < appdata.column_patterns->len; i++)
+    {
+      if (node_matches_spec_list(node, g_ptr_array_index(appdata.column_patterns, i)))
+        return i;
+    }
+
+  /*
+   * If no explicit match was found it goes in the rightmost column (with an
+   * implicit "match-all" pattern).
+   */
+  return appdata.column_patterns->len;
+}
+
+/*
+ * A preparatory pass to count how many nodes are centered and how many are on
+ * the outer ring (and mark each appropriately).  Also does analogous work for
+ * columnar-positioning mode (count nodes in each column and mark each node with
+ * its column).
+ */
+static gint reposition_canvas_nodes_prep(const node_id_t *node_id,
+                                         canvas_node_t *canvas_node,
+                                         reposition_node_t *rdata)
+{
+  node_t *node;
+
+  if (!canvas_node->shown)
+    return FALSE;
+
+  node = nodes_catalog_find(node_id);
+  if (appdata.column_patterns)
+    {
+      canvas_node->column = find_node_column(node);
+      canvas_node->column_idx = rdata->column_populations[canvas_node->column]++;
+    }
+  else if (node && node_matches_spec_list(node, centered_node_speclist))
+    {
+      canvas_node->centered = TRUE;
+      rdata->center.n_nodes++;
+    }
+  else
+    {
+      canvas_node->centered = FALSE;
+      rdata->outer.n_nodes++;
+    }
+
+  return FALSE;
+}
+
+/*
+ * Return a point between 'min' and 'max' appropriate for position number 'pos'
+ * out of 'num' total possible positions (basically pos/num of the way between
+ * min and max, though with some tweaking to keep things away from the very
+ * edges of the range).
+ */
+static gdouble scale_within(gdouble min, gdouble max, guint pos, guint num)
+{
+  return min + (((max - min) / (num + 1)) * (pos + 1));
+}
+
 /* Called from update_diagram if the global need_reposition
  * is set. It rearranges the nodes*/
 /* TODO I think I should update all links as well, so as not having
@@ -957,8 +1293,9 @@ static gint reposition_canvas_nodes(node_id_t * node_id,
                                     canvas_node_t * canvas_node,
                                     reposition_node_t *data)
 {
+  struct node_ring *ring;
+  gdouble center_x, center_y, oddAngle;
   gdouble x = 0, y = 0;
-  gdouble oddAngle;
 
   if (!canvas_node->shown)
     {
@@ -967,11 +1304,14 @@ static gint reposition_canvas_nodes(node_id_t * node_id,
       return FALSE;
     }
 
-  oddAngle = data->angle;
-  
+  ring = canvas_node->centered ? &data->center : &data->outer;
+
+  center_x = (data->xmax - data->xmin) / 2 + data->xmin;
+  center_y = (data->ymax - data->ymin) / 2 + data->ymin;
+
   /* TODO I've done all the stationary changes in a hurry
    * I should review it an tidy up all this stuff */
-  if (pref.stationary)
+  if (appdata.stationary_layout)
     {
       if (canvas_node->is_new)
 	{
@@ -998,46 +1338,53 @@ static gint reposition_canvas_nodes(node_id_t * node_id,
 	  y = data->y_rad_max * sin (s_angle);
 	}
     }
+  else if (appdata.column_patterns)
+    {
+      guint col = canvas_node->column;
+
+      x = scale_within(data->xmin, data->xmax, canvas_node->column,
+                       appdata.column_patterns->len + 1);
+      y = scale_within(data->ymin, data->ymax, canvas_node->column_idx,
+                       data->column_populations[col]);
+    }
   else
     {
-      if (canvas_node->is_new && *pref.center_node) {
-        node_t * node;
-
-        /* center node specified, check to see if current node is centered */  
-        node = nodes_catalog_find((const node_id_t *)&canvas_node->canvas_node_id);
-        /* If it is one of the "centered" nodes change the coordinates */
-        if (node) {
-          if (!strcmp(node->name->str, pref.center_node) ||
-              !strcmp(node->numeric_name->str, pref.center_node)) {
-             canvas_node->centered = TRUE;
-          }
-        }
-      }
-
-      if (canvas_node->centered) {
-         /* centered node, reset coordinates */
-         x = ( data->xmax - data->xmin ) / 2 + data->xmin ;
-         y = ( data->ymax - data->ymin ) / 2 + data->ymin ;
-         data->angle -= 2 * M_PI / data->n_nodes;
-      }
-      else {
-        if (data->n_nodes % 2 == 0)	/* spacing is better when n_nodes is odd and Y is linear */
-            oddAngle = (data->angle * data->n_nodes) / (data->n_nodes + 1);
-        if (data->n_nodes > 7)
+      if (canvas_node->centered && data->center.n_nodes == 1)
         {
-          x = data->x_rad_max * cos (oddAngle);
-          y = data->y_rad_max * asin (sin (oddAngle)) / (M_PI / 2);
+          /* one centered node, reset coordinates */
+          x = center_x;
+          y = center_y;
+          ring->angle -= 2 * M_PI / ring->n_nodes;
         }
-        else
+      else
         {
-          x = data->x_rad_max * cos (data->angle);
-          y = data->y_rad_max * sin (data->angle);
+          if (ring->n_nodes % 2 == 0)	/* spacing is better when n_nodes is odd and Y is linear */
+            oddAngle = (ring->angle * ring->n_nodes) / (ring->n_nodes + 1);
+          else
+            oddAngle = ring->angle;
+
+          if (ring->n_nodes > 7)
+            {
+              x = data->x_rad_max * cos (oddAngle);
+              y = data->y_rad_max * asin (sin (oddAngle)) / (M_PI / 2);
+            }
+          else
+            {
+              x = data->x_rad_max * cos (ring->angle);
+              y = data->y_rad_max * sin (ring->angle);
+            }
+
+          if (canvas_node->centered && data->center.n_nodes > 1)
+            {
+              /* For the inner ring, just move it proportionally closer the the center point. */
+              x = center_x + ((x - center_x) * pref.inner_ring_scale);
+              y = center_y + ((y - center_y) * pref.inner_ring_scale);
+            }
         }
-      }
     }
 
 
-  if (!pref.stationary || canvas_node->is_new)
+  if (!appdata.stationary_layout || canvas_node->is_new)
     {
       gnome_canvas_item_set(GNOME_CANVAS_ITEM (canvas_node->group_item),
 			     "x", x, "y", y, NULL);
@@ -1066,14 +1413,14 @@ static gint reposition_canvas_nodes(node_id_t * node_id,
   gnome_canvas_item_show (canvas_node->node_item);
   gnome_canvas_item_request_update (canvas_node->node_item);
 
-  data->node_i--;
+  ring->node_i--;
 
-  if (data->node_i)
-    data->angle += 2 * M_PI / data->n_nodes;
+  if (ring->node_i)
+    ring->angle += 2 * M_PI / ring->n_nodes;
   else
     {
-      data->angle = 0.0;
-      data->n_nodes = 0;
+      ring->angle = 0.0;
+      ring->n_nodes = 0;
     }
 
   return FALSE;
@@ -1186,13 +1533,13 @@ canvas_link_update(link_id_t * link_id, canvas_link_t * canvas_link,
       canvas_link->color = protohash_color(link->main_prot[pref.stack_level]);
 
       /* scale color down to 10% at link timeout */
-      diffms = substract_times_ms(&appdata.now, &link->link_stats.stats.last_time);
+      diffms = subtract_times_ms(&appdata.now, &link->link_stats.stats.last_time);
       scale = pow(0.10, diffms / pref.gui_link_timeout_time);
 
       scaledColor =
-        (((int) (scale * canvas_link->color.red) & 0xFF00) << 16) |
-        (((int) (scale * canvas_link->color.green) & 0xFF00) << 8) |
-        ((int) (scale * canvas_link->color.blue) & 0xFF00) | 0xFF;
+        (((guint32) (scale * canvas_link->color.red) & 0xFF00) << 16) |
+        (((guint32) (scale * canvas_link->color.green) & 0xFF00) << 8) |
+        ((guint32) (scale * canvas_link->color.blue) & 0xFF00) | 0xFF;
     }
   else
     {
@@ -1407,6 +1754,29 @@ node_item_event (GnomeCanvasItem * item, GdkEvent * event,
 
 }				/* node_item_event */
 
+/* Explain pcap stats in status bar when moused over */
+static gint pcap_stats_text_item_event(GnomeCanvasItem *item, GdkEvent *event,
+                                       void *unused)
+{
+  switch (event->type)
+    {
+    case GDK_ENTER_NOTIFY:
+      gtk_statusbar_push(appdata.statusbar, 1, _("'recv': packets received; "
+                                                 "'drop': packets dropped by OS buffering; "
+                                                 "'ifdrop': packets dropped by interface or driver."));
+      break;
+
+    case GDK_LEAVE_NOTIFY:
+      gtk_statusbar_pop(appdata.statusbar, 1);
+      break;
+
+    default:
+      break;
+    }
+
+  return FALSE;
+}
+
 /* Pushes a string into the statusbar stack */
 void
 set_statusbar_msg (gchar * str)
