@@ -17,12 +17,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#ifndef DIAGRAM_H
+#define DIAGRAM_H
+
 #include "appdata.h"
 
-
-
-gboolean already_updating;	/* True while an instance of update_diagram is running */
-gboolean stop_requested;	/* True if there is a pending stop request */
+extern gboolean already_updating; /* True while an instance of update_diagram is running */
+extern gboolean stop_requested; /* True if there is a pending stop request */
 
 guint update_diagram (GtkWidget * canvas);
 void init_diagram (GladeXML *xml);
@@ -33,3 +34,6 @@ void delete_gui_protocols (void);
 void ask_reposition(gboolean refresh_font); /* request diagram relayout */
 void dump_stats(guint32 diff_msecs);
 void timeout_changed(void);
+void redraw_canvas_background(GtkWidget *canvas); /* Called by on_canvas1_size_allocate */
+
+#endif /* DIAGRAM_H */
