@@ -26,13 +26,14 @@
 #include "appdata.h"
 #include "info_windows.h"
 #include "diagram.h"
-#include "node.h"
+#include "stats/node.h"
 #include "datastructs.h"
-#include "protocols.h"
-#include "capture.h"
+#include "stats/protocols.h"
+#include "capture/capctl.h"
 #include "preferences.h"
-#include "prot_types.h"
+#include "stats/prot_types.h"
 #include "ui_utils.h"
+#include "stats/util.h"
 #include "node_windows.h"
 
 typedef enum 
@@ -210,7 +211,7 @@ update_prot_info_window (prot_info_window_t * prot_info_window)
   widget = g_object_get_data (G_OBJECT (window), "name_label");
   gtk_label_set_text (GTK_LABEL (widget), prot_info_window->prot_name);
   widget = g_object_get_data (G_OBJECT (window), "last_heard_label");
-  str = timeval_to_str (prot->stats.last_time);
+  str = timeval_to_str(prot->stats.last_time);
   gtk_label_set_text (GTK_LABEL (widget), str);
   g_free(str);
   widget = g_object_get_data (G_OBJECT (window), "average");
@@ -231,7 +232,7 @@ update_prot_info_windows (void)
 {
   GList *list_item = NULL, *remove_item;
   prot_info_window_t *prot_info_window = NULL;
-  enum status_t status;
+  capstatus_t status;
 
   status = get_capture_status();
 
@@ -331,7 +332,7 @@ protocols_table_compare (GtkTreeModel * gs, GtkTreeIter * a, GtkTreeIter * b,
 	ret = 1;
       break;
     case PROTO_COLUMN_LASTHEARD:
-      diffms = substract_times_ms(&prot1->rowstats.last_time, 
+      diffms = subtract_times_ms(&prot1->rowstats.last_time, 
                                   &prot2->rowstats.last_time);
       if (diffms == 0)
 	ret = 0;
@@ -677,14 +678,12 @@ node_info_window_compare(gconstpointer a, gconstpointer b)
 guint
 update_info_windows (void)
 {
-  enum status_t status;
+  capstatus_t status;
 
   status = get_capture_status();
   if (status != PLAY && status != STOP)
     return TRUE;
 
-  gettimeofday (&appdata.now, NULL);
-
   update_protocols_window ();
   update_stats_info_windows ();
   update_prot_info_windows ();
@@ -717,7 +716,7 @@ update_stats_info_windows (void)
 {
   GList *list_item;
   GtkWidget *window;
-  enum status_t status;
+  capstatus_t status;
   double diffms;
   static struct timeval last_update_time = {
     0, 0
@@ -726,7 +725,7 @@ update_stats_info_windows (void)
   status = get_capture_status();
 
   /* Update info windows at most twice a second */
-  diffms = substract_times_ms(&appdata.now, &last_update_time);
+  diffms = subtract_times_ms(&appdata.now, &last_update_time);
   if (pref.refresh_period < 500)
     if (diffms < 500)
       return;
