@@ -24,9 +24,10 @@
 #include "pref_dialog.h"
 #include "preferences.h"
 #include "diagram.h"
-#include "capture.h"
+#include "capture/capctl.h"
+#include "stats/node.h"
 #include "datastructs.h"
-#include "util.h" /* compat functions */
+#include "compat.h"
 
 #define MILLI   (1000.0)
 #define COLSPACES   "        "
@@ -45,7 +46,7 @@ static void cbox_add_select(GtkComboBoxEntry *cbox, const gchar *str);
 
 static gboolean colors_changed = FALSE;
 static GtkWidget *diag_pref = NULL;  /* diagram configuration window */
-static struct pref_struct *tmp_pref = NULL; /* tmp copy of pref data */
+static struct pref_struct tmp_pref; /* tmp copy of pref data */
 
 
 static void
@@ -70,9 +71,7 @@ static void
 hide_pref_dialog(void)
 {
   /* purge temporary */
-  free_config(tmp_pref);
-  g_free(tmp_pref);
-  tmp_pref = NULL;
+  free_config(&tmp_pref);
 
   /* reset flags */
   colors_changed = FALSE;
@@ -102,6 +101,11 @@ initialize_pref_controls(void)
 			    pref.link_node_ratio);
   g_signal_emit_by_name (GTK_OBJECT (GTK_RANGE (widget)->adjustment),
 			 "changed");
+  widget = glade_xml_get_widget (appdata.xml, "inner_ring_scale_slider");
+  gtk_adjustment_set_value (GTK_RANGE (widget)->adjustment,
+                            pref.inner_ring_scale);
+  g_signal_emit_by_name (GTK_OBJECT (GTK_RANGE (widget)->adjustment),
+			 "changed");
   spin = GTK_SPIN_BUTTON (glade_xml_get_widget (appdata.xml, "averaging_spin"));
   gtk_spin_button_set_value (spin, pref.averaging_time);
   spin = GTK_SPIN_BUTTON (glade_xml_get_widget (appdata.xml, "refresh_spin"));
@@ -143,6 +147,9 @@ initialize_pref_controls(void)
     gdk_color_parse("#ffff00", &color);
   gtk_color_button_set_color(GTK_COLOR_BUTTON(widget), &color);
 
+  widget = glade_xml_get_widget (appdata.xml, "pcap_stats_pos");
+  gtk_combo_box_set_active(GTK_COMBO_BOX(widget), pref.pcap_stats_pos);
+
   widget = glade_xml_get_widget (appdata.xml, "filter_combo");
   model = gtk_combo_box_get_model(GTK_COMBO_BOX(widget));
   if (!model)
@@ -163,6 +170,15 @@ initialize_pref_controls(void)
 
   pref_to_color_list();		/* Updates the color preferences table with pref.colors */
 
+  widget = glade_xml_get_widget(appdata.xml, "bck_image_enabled");
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), pref.bck_image_enabled);
+  gtk_widget_set_sensitive(glade_xml_get_widget(appdata.xml, "bck_image_open"), pref.bck_image_enabled);
+
+  if (pref.bck_image_path && *pref.bck_image_path) {
+    widget = glade_xml_get_widget(appdata.xml, "bck_image_open");
+    gtk_file_chooser_select_filename(GTK_FILE_CHOOSER(widget), pref.bck_image_path);
+  }
+
   /* Connects signals */
   widget = glade_xml_get_widget (appdata.xml, "diag_pref");
   g_signal_connect (G_OBJECT (widget),
@@ -179,6 +195,11 @@ initialize_pref_controls(void)
 		    "value_changed",
 		    GTK_SIGNAL_FUNC
 		    (on_link_width_slider_adjustment_changed), NULL);
+  widget = glade_xml_get_widget (appdata.xml, "inner_ring_scale_slider");
+  g_signal_connect (G_OBJECT (GTK_RANGE (widget)->adjustment),
+		    "value_changed",
+		    GTK_SIGNAL_FUNC
+		    (on_inner_ring_scale_slider_adjustment_changed), NULL);
   widget = glade_xml_get_widget (appdata.xml, "averaging_spin");
   g_signal_connect (G_OBJECT (GTK_SPIN_BUTTON (widget)->adjustment),
 		    "value_changed",
@@ -253,15 +274,15 @@ on_preferences1_activate (GtkMenuItem * menuitem, gpointer user_data)
   GtkComboBoxEntry *cbox;
 
   /* saves current prefs to a temporary */
-  tmp_pref = duplicate_config(&pref);
+  copy_config(&tmp_pref, &pref);
 
   initialize_pref_controls();
-  
+
   cbox = GTK_COMBO_BOX_ENTRY(glade_xml_get_widget (appdata.xml, "filter_combo"));
   cbox_add_select(cbox, pref.filter);
 
   cbox = GTK_COMBO_BOX_ENTRY(glade_xml_get_widget (appdata.xml, "center_combo"));
-  cbox_add_select(cbox, pref.center_node);
+  cbox_add_select(cbox, pref.centered_nodes);
 
   gtk_widget_show (diag_pref);
   gdk_window_raise (diag_pref->window);
@@ -289,6 +310,16 @@ on_link_width_slider_adjustment_changed (GtkAdjustment * adj)
 
 }
 
+void
+on_inner_ring_scale_slider_adjustment_changed (GtkAdjustment * adj)
+{
+  pref.inner_ring_scale = adj->value;
+  g_log (G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG,
+         _("Adjustment value: %g. Inner ring scale %g"),
+         adj->value, pref.inner_ring_scale);
+  ask_reposition(FALSE);
+}
+
 void
 on_averaging_spin_adjustment_changed (GtkAdjustment * adj)
 {
@@ -415,7 +446,7 @@ void
 on_cancel_pref_button_clicked (GtkButton * button, gpointer user_data)
 {
   /* reset configuration to saved */
-  copy_config(&pref, tmp_pref);
+  copy_config(&pref, &tmp_pref);
 
   ask_reposition(TRUE);
   if (colors_changed)
@@ -431,7 +462,7 @@ void
 on_save_pref_button_clicked (GtkButton * button, gpointer user_data)
 {
   confirm_changes();	/* to save we simulate confirmation */
-  save_config ();
+  save_config(&pref);
   hide_pref_dialog();
 }				/* on_save_pref_button_clicked */
 
@@ -448,7 +479,7 @@ static void on_filter_entry_changed(GtkComboBoxEntry * cbox, gpointer user_data)
   pref.filter = g_strdup (str);
   /* TODO We should look at the error code from set_filter and pop
    * up a window accordingly */
-  set_filter (pref.filter, NULL);
+  set_filter(pref.filter);
   g_free (str);
   cbox_add_select(cbox, pref.filter);
 }				/* on_filter_entry_changed */
@@ -458,10 +489,13 @@ static void on_center_node_changed(GtkComboBoxEntry * cbox, gpointer user_data)
 {
   gchar *str;
   str = gtk_combo_box_get_active_text(GTK_COMBO_BOX(cbox));
-  g_free(pref.center_node);
-  pref.center_node = g_strdup (str);
-  g_free (str);
-  cbox_add_select(cbox, pref.center_node);
+  g_free(pref.centered_nodes);
+  pref.centered_nodes = g_strdup(str);
+  g_free(str);
+  free_nodeset_spec_list(centered_node_speclist);
+  centered_node_speclist = parse_nodeset_spec_list(pref.centered_nodes);
+  cbox_add_select(cbox, pref.centered_nodes);
+  ask_reposition(FALSE);
 }
 
 void
@@ -470,6 +504,25 @@ on_numeric_toggle_toggled (GtkToggleButton * togglebutton, gpointer user_data)
   pref.name_res = gtk_toggle_button_get_active (togglebutton);
 }				/* on_numeric_toggle_toggled */
 
+void 
+on_background_image_path_selected (GtkFileChooserButton * filechooser, gpointer user_data)
+{
+  pref.bck_image_path = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER (filechooser));
+}       /* on_background_image_path_selected */
+
+void
+on_background_image_enable_toggled(GtkToggleButton *togglebutton, gpointer user_data)
+{
+  gboolean on = gtk_toggle_button_get_active(togglebutton);
+  pref.bck_image_enabled = on;
+  gtk_widget_set_sensitive(glade_xml_get_widget(appdata.xml, "bck_image_open"), on);
+}
+
+void on_pcap_stats_display_changed(GtkComboBox *combobox, gpointer user_data)
+{
+  pref.pcap_stats_pos = gtk_combo_box_get_active(combobox);
+}
+
 /* ----------------------------------------------------------
 
    Color-proto preferences handling
@@ -631,7 +684,7 @@ on_colordiag_ok_clicked (GtkButton * button, gpointer user_data)
           if (!gtk_tree_model_get_iter
               (GTK_TREE_MODEL (ep.gs), &itsibling, gpath))
             return;			/* path not found */
-            gtk_list_store_insert_before (ep.gs, &it, &itsibling);
+          gtk_list_store_insert_before (ep.gs, &it, &itsibling);
         }
       else
         gtk_list_store_append (ep.gs, &it);	/* no row selected, append */
